
// dlgShapeScript.cpp: implementation of the dlgShapeScript class.
//
//////////////////////////////////////////////////////////////////////

#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include <qtabwidget.h>
#include <qlineedit.h>
#include <qmessagebox.h>
#include <qstring.h>
#include <qfiledialog.h>
#include <qmultilineedit.h>

#include "dlgShapeScript.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

dlgShapeScript:: dlgShapeScript(KrosseyeDoc * doc, QWidget * parent, const char *name):dlgShapeScript_ui(parent, name, false)
{
	Doc = doc;
	timer = new QTimer(this);
	exec_line = -1;
	offsetx = 0;
	offsety = 0;
	scale = 100;

	connect(timer, SIGNAL(timeout()), this, SLOT(slotTick()));

}

dlgShapeScript::~dlgShapeScript()
{

}



int dlgShapeScript::getAttr(QString line, QString tag, int def)
{
	int pos, len;
	QString err;

	pos = line.find(tag);

	if (pos < 0)
	{
		if (def != NO_DEF)
			return def;

		err = "Cannot find the required tag " + tag + " in line \n\n'" + line + "'";
	  QMessageBox::warning(this, "Krosseye", err);
	}

	pos += tag.length() + 1;	/* +1 for the '=' sign */

	for (len = 0; len < 5; len++)
	{
		if (line.at(pos + len) == ' ')
			break;
	}

	return (line.mid(pos, len).toInt());
}

#define ADJ(x)  (((x)*scale)/100)
#define ADJX(x) (ADJ(x)+cw+offsetx)
#define ADJY(x) (ADJ(x)+ch+offsety)

bool dlgShapeScript::do_exec_line(int line_to_exec)
{
	int cw = Doc->width / 2;
	int ch = Doc->heigth / 2;

	int lineend, i;
	QString line, err, s = exec_script;


/*** FINE LINE TO RUN ***/

	for (i = 0; i < line_to_exec; i++)
	{
		/* Extract a line */
		lineend = s.find("\n");
		line = s.left(lineend);
		s = s.mid(lineend + 1);
	}

/*** ARE WE DONE? ***/

	if (s.find('\n') < 0)
	{
		/* Stop exec */
		return (true);
	}

/*** EXEC THE LINE ***/

	/* ignore comments */
	if (line.length() < 2)
		return (false);
	if (line.left(1) == "-")
		return (false);

	/* Check for commands */
	if (line.find("CLEAR") >= 0)
	{
		Doc->clearDepthmap();
		return (false);
	}

	if (line.find("DOT") >= 0)
	{
		int cx, cy, cz, size;

		cx = ADJX(getAttr(line, "CX"));
		cy = ADJY(getAttr(line, "CY"));
		cz = getAttr(line, "CZ");
		size = ADJ(getAttr(line, "SIZE", 1));
		if (size > 1000)
			size = 1000;

		Doc->drawDepthmap_dot(cx, cy, cz, size);
		Doc->ForceDocumentChanged();
		return (false);
	}

	if (line.find("RAISE") >= 0)
	{
		int cx, cy, size, dist, pow;

		cx = ADJX(getAttr(line, "CX"));
		cy = ADJY(getAttr(line, "CY"));
		dist = getAttr(line, "DIST", 0);
		pow = getAttr(line, "POW", 0);
		size = ADJ(getAttr(line, "SIZE", 1));
		if (size > 1000)
			size = 1000;

		Doc->drawDepthmap_raise(cx, cy, size, dist, pow);
		Doc->ForceDocumentChanged();
		return (false);
	}

	if (line.find("RLINE") >= 0)
	{
		int ax, ay, az, bx, by, bz, size;

		ax = ADJX(getAttr(line, "AX"));
		ay = ADJY(getAttr(line, "AY"));
		az = getAttr(line, "AZ");
		bx = ADJX(getAttr(line, "BX"));
		by = ADJY(getAttr(line, "BY"));
		bz = getAttr(line, "BZ");
		size = ADJ(getAttr(line, "SIZE", 1));

		if (size > 200)
			size = 200;

		Doc->drawDepthmap_flatline(ax, ay, az, bx, by, bz, size);
		Doc->ForceDocumentChanged();
		return (false);
	}

	if (line.find("LINE") >= 0)
	{
		int ax, ay, az, bx, by, bz, size, pow;

		ax = ADJX(getAttr(line, "AX"));
		ay = ADJY(getAttr(line, "AY"));
		az = getAttr(line, "AZ");
		bx = ADJX(getAttr(line, "BX"));
		by = ADJY(getAttr(line, "BY"));
		bz = getAttr(line, "BZ");
		size = ADJ(getAttr(line, "SIZE", 1));
		pow = getAttr(line, "POW", 0);
		if (size > 200)
			size = 200;

		Doc->drawDepthmap_line(ax, ay, az, bx, by, bz, size, pow);
		Doc->ForceDocumentChanged();
		return (false);
	}


	if (line.find("SOFTEN") >= 0)
	{
		int cx, cy, size, pow;

		cx = ADJX(getAttr(line, "CX"));
		cy = ADJY(getAttr(line, "CY"));
		size = ADJ(getAttr(line, "SIZE", 10));
		pow = getAttr(line, "POW", 20);
		if (size > 200)
			size = 200;

		Doc->drawDepthmap_soften(cx, cy, size, pow);
		Doc->ForceDocumentChanged();
		return (false);
	}

	if (line.find("TRIANGLE") >= 0)
	{
		int cx, cy, cz;
		int ax, ay, az;
		int bx, by, bz;

		ax = ADJX(getAttr(line, "AX"));
		ay = ADJY(getAttr(line, "AY"));
		az = getAttr(line, "AZ");

		bx = ADJX(getAttr(line, "BX"));
		by = ADJY(getAttr(line, "BY"));
		bz = getAttr(line, "BZ");

		cx = ADJX(getAttr(line, "CX"));
		cy = ADJY(getAttr(line, "CY"));
		cz = getAttr(line, "CZ");


		Doc->drawDepthmap_triangle(ax, ay, az, bx, by, bz, cx, cy, cz);
		Doc->ForceDocumentChanged();
		return (false);
	}

	if (line.find("CONE") >= 0)
	{
		int cx, cy, cz, pow;
		int size;

		cx = ADJX(getAttr(line, "CX"));
		cy = ADJY(getAttr(line, "CY"));
		cz = getAttr(line, "CZ");

		pow = getAttr(line, "POW", 100);
		size = ADJ(getAttr(line, "SIZE"));
		if (size > 200)
			size = 200;

		Doc->drawDepthmap_cone(cx, cy, cz, pow, size);
		Doc->ForceDocumentChanged();
		return (false);
	}

	if (line.find("SPHERE") >= 0)
	{
		int cx, cy, cz;
		int w, h, pow, arg;

		cx = ADJX(getAttr(line, "CX"));
		cy = ADJY(getAttr(line, "CY"));
		cz = getAttr(line, "CZ");


		w = getAttr(line, "SIZEX");
		h = ADJ(getAttr(line, "SIZEY", w));
		pow = getAttr(line, "POW", 100);

		w = ADJ(w);

		arg = getAttr(line, "ARG", 0);

		if (w > 500)
			w = 500;
		if (h > 500)
			h = 500;

		Doc->drawDepthmap_sphere(cx, cy, cz, w, h, pow, arg);
		Doc->ForceDocumentChanged();
		return (false);
	}


	if (line.find("DISC") >= 0)
	{
		int cx, cy, cz, rad, arg1, arg2;

		rad = ADJ(getAttr(line, "RAD"));
		if (rad > 500)
			rad = 500;

		arg1 = getAttr(line, "ARG1", 0);	/* Gradient angle */
		arg2 = getAttr(line, "ARG2", 0);	/* Gradient strength */


		cx = ADJX(getAttr(line, "CX"));
		cy = ADJY(getAttr(line, "CY"));
		cz = getAttr(line, "CZ");

		Doc->drawDepthmap_disc(cx, cy, cz, rad, arg1, arg2);
		Doc->ForceDocumentChanged();
		return (false);
	}


	if ((line.find("COPY") >= 0) || (line.find("TILT") >= 0))
	{
		int sx, sy, dx, dy, sizex, sizey, argx, argy, argz;


		argx = getAttr(line, "ARGX", 0);
		argy = getAttr(line, "ARGY", 0);
		argz = getAttr(line, "ARGZ", 0);

		sx = getAttr(line, "SX");
		sy = getAttr(line, "SY");

		dx = ADJX(getAttr(line, "DX", sx));
		dy = ADJY(getAttr(line, "DY", sy));

		sx = ADJX(sx);
		sy = ADJY(sy);

		sizex = getAttr(line, "SIZEX");
		sizey = getAttr(line, "SIZEY", sizex);

		sizex = ADJ(sizex);
		sizey = ADJ(sizey);

		if (sizex > 500)
			sizex = 500;

		if (sizey > 500)
			sizey = 500;

		Doc->drawDepthmap_copy(sx, sy, sizex, sizey, dx, dy, argx, argy, argz);
		Doc->ForceDocumentChanged();
		return (false);
	}



	if (line.find("LOAD") >= 0)
	{
		QString filename;
		int cx, cy, cz, sizex, sizey, arg;

		int pos = line.find("LOAD");

		cx = ADJX(getAttr(line, "CX", 0));
		cy = ADJY(getAttr(line, "CY", 0));
		cz = getAttr(line, "CZ", 0);
		sizex = ADJ(getAttr(line, "SIZEX", Doc->width));
		sizey = ADJ(getAttr(line, "SIZEY", Doc->heigth));
		arg = getAttr(line, "ARG", 0);

		filename = line.mid(pos + 5);
		filename = filename.left(filename.find(" "));

		if (sizex > Doc->width)
			sizex = Doc->width;

		if (sizey > Doc->heigth)
			sizey = Doc->heigth;

		Doc->drawDepthmap_load(filename, cx, cy, cz, sizex, sizey, arg);
		Doc->ForceDocumentChanged();

		return (false);
	}


	if (line.find("SET") >= 0)
	{
		int mode;

		offsetx = getAttr(line, "OFFSETX", offsetx);
		offsety = getAttr(line, "OFFSETY", offsety);

		scale = getAttr(line, "SCALE", scale);
		if (scale <= 0)
			scale = 1;
		if (scale > 1000)
			scale = 1000;

		mode = getAttr(line, "MODE", 0);

		switch (mode)
		{
			default:
			case 0:
			  Doc->depthmap_drawing_mode = KrosseyeDoc::DEPD_NORMAL;
				break;
			case 1:
			  Doc->depthmap_drawing_mode = KrosseyeDoc::DEPD_RAISE;
				break;
			case 2:
			  Doc->depthmap_drawing_mode = KrosseyeDoc::DEPD_LOWER;
				break;
			case 3:
			  Doc->depthmap_drawing_mode = KrosseyeDoc::DEPD_CUTBELOW;
				break;
/*          case 4:
   Doc->depthmap_drawing_mode = KrosseyeDoc::DEPD_CUTBELOW;
   break;
 */
		}
		return (false);
	}



/*** Make sure that the user gets annoyed about all his/hers typos ***/

	err = "Cannot parse the following line:\n\n'" + line + "'\n\n(Skipping it)";
  QMessageBox::warning(this, "Krosseye", err);

	return (false);
}

void dlgShapeScript::slotRunScript()
{
	if (exec_line >= 0)
	{
		do_exec_stop();
		return;
	}



	exec_script = txtScript->text();
	exec_script += "\n";
	exec_line = 0;

	/* Start the timer */
	timer->start(0, FALSE);
}

void dlgShapeScript::do_exec_stop(void)
{
	exec_line = -1;
	timer->stop();

	/* Reset data */
	scale = 100;
	offsetx = 0;
	offsety = 0;
  Doc->depthmap_drawing_mode = KrosseyeDoc::DEPD_NORMAL;
}

void
 dlgShapeScript::slotTick()
{
	if (exec_line >= 0)
	{
		if (do_exec_line(exec_line++))
		{
			do_exec_stop();
		}
	}
}

void dlgShapeScript::slotLoadScript()
{

	FILE *f;
	char c;

	QString s(QFileDialog::getOpenFileName(QString::null, "*.kds", this));
	if (s.isEmpty())
		return;


	f = fopen(s, "rt");
	if (!f)
	{
	  QMessageBox::information(this, "Krosseye", "Unable to open that file");
		return;
	}

	s = "";
	while (!feof(f))
	{
		c = fgetc(f);
		if (!feof(f))
			s.append(c);
	}
	fclose(f);

	txtScript->setText(s);

}

void dlgShapeScript::slotSaveScript()
{
	FILE *f;


	QString s(QFileDialog::getSaveFileName(QString::null, "*.kds", this));
	if (s.isEmpty())
		return;

	s = s.lower();

	if (s.find(".kds") < 0)
	{
		s.append(".kds");
	}


	f = fopen(s, "wt");
	if (!f)
	{
	  QMessageBox::information(this, "Krosseye", "Unable to open that file");
		return;
	}


	fprintf(f, txtScript->text());

	fclose(f);

}
