/***************************************************************************
                          krosseye.cpp  -  description
                             -------------------
    begin                : Sun Jun 10 19:09:16 CEST 2001
    copyright            : (C) 2001 by Tersevs
    email                : tersevs@home.se
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <qaccel.h>
#include "krosseye.h"
#include "filesave.xpm"
#include "fileopen.xpm"
#include "filenew.xpm"
#include "dlgnew.h"
#include "dlgabout.h"
#include "dlgShapeScript.h"


#ifdef WIN32
#define SUPPORTED_IMG_FORMATS	"Images (*.png *.xpm *.bmp)"
#define VERSION_CAPTION			"Krosseye 0.3"
#else
#define SUPPORTED_IMG_FORMATS	"Images (*.png *.xpm *.jpg *.bmp)"
#endif

/* XPM */
/*static char * krosseye_xpm[] = { */
static const char *krosseye_xpm[] =
{
	"32 32 10 1",
	" 	c None",
	".	c #303030",
	"+	c #585858",
	"@	c #C3C3C3",
	"#	c #FFFFFF",
	"$	c #FF0000",
	"%	c #000080",
	"&	c #58A8FF",
	"*	c #0000C0",
	"=	c #0080FF",
	"                                ",
	"                                ",
	"                                ",
	"                                ",
	"       ....           ....      ",
	"      .++++.         .++++.     ",
	"     .+@@@@+.       .+@@@@+.    ",
	"    .+@####@+.     .+@$##$@+.   ",
	"    .+@###@@@+     .+@##$##$+   ",
	"   .+@$##@..@+.   .+@#####$@$.  ",
	"   .+@##@.%%.+.   .+@#######@.  ",
	"  .+$##@.%%%%.+. .+@##@@##$#$+. ",
	"  .+@##@.&*&*.+. .+@#@..@##$@+. ",
	"  .+@$#@.&&*%.+. .+@@.%%.@##$+. ",
	"  .+@##@.&*&*.+. .+@.%%%%.@$@+. ",
	"  .+$#$@.&*=*.+. .+@.&*&*.@#@+. ",
	"  .+@##@.%%%.@+. .+@.&&*%.@#$+. ",
	"  .+@$##@.%%.@+. .+@.&*&*.@$@+. ",
	"  .+$####@..@@+. .+@.&*=%.@#@+. ",
	"  .+@@$###@@#@+. .+@.%%%.@##@+. ",
	"   .+$######@+.   .+@.%%.@#@+.  ",
	"   .+@$#####@+.   .+@@..@##@+.  ",
	"    +$@$###@+.     +@#@@##@+.   ",
	"    .+@####@+.     .+@####@+.   ",
	"     .+@@@@+.       .+@@@@+.    ",
	"      .++++.         .++++.     ",
	"       ....           ....      ",
	"                                ",
	"                                ",
	"                                ",
	"                                ",
	"                                "};



KrosseyeApp::KrosseyeApp()
{
#ifdef VERSION
	setCaption(tr("Krosseye " VERSION));
#else
	setCaption(tr(VERSION_CAPTION));
#endif
	initActions();
	initMenuBar();
	initToolBar();
	initStatusBar();

	initDoc();
	initView();
	initDialogs();

	pixIcon = new QPixmap(krosseye_xpm);
	setIcon(*pixIcon);

	viewToolBar->setOn(false);
	viewStatusBar->setOn(true);
	viewDebug->setOn(false);

	fileToolbar->hide();
	//  statusBar()->hide();
	//
	connect(doc, SIGNAL(signalPercentDone(int)), this,
			SLOT(slotProcessing(int)));

	connect(view, SIGNAL(signalMousePos(int, int)), this,
			SLOT(slotMousePos(int, int)));

}

KrosseyeApp::~KrosseyeApp()
{
	delete pixIcon;
}

/** initializes all QActions of the application */
void
 KrosseyeApp::initActions()
{

	QPixmap openIcon, saveIcon, newIcon;
	newIcon = QPixmap(filenew);
	openIcon = QPixmap(fileopen);
	saveIcon = QPixmap(filesave);

	fileNew =
		new QAction(tr("New File"), newIcon, tr("&New..."),
  QAccel::		stringToKey(tr("Ctrl+N")), this);
	fileNew->setStatusTip(tr("Creates a new Stereogram with a new size"));
	fileNew->setWhatsThis(tr("New File\n\nCreates a new stereogram with a new size"));
	connect(fileNew, SIGNAL(activated()), this, SLOT(slotFileNew()));

/*  filePrint = new QAction(tr("Print File"), tr("&Print"), QAccel::stringToKey(tr("Ctrl+P")), this);
   filePrint->setStatusTip(tr("Prints out the actual document"));
   filePrint->setWhatsThis(tr("Print File\n\nPrints out the actual document"));
   connect(filePrint, SIGNAL(activated()), this, SLOT(slotFilePrint()));
 */

	fileQuit =
		new QAction(tr("Exit"), tr("E&xit"),
  QAccel::		stringToKey(tr("Ctrl+Q")), this);
	fileQuit->setStatusTip(tr("Quits the application"));
	fileQuit->setWhatsThis(tr("Exit\n\nQuits the application"));
	connect(fileQuit, SIGNAL(activated()), this, SLOT(slotFileQuit()));

	ClearStereogram = new QAction(tr("Clear all layers"), tr("&Clear Stereogram"), 0, this);
	ClearStereogram->setStatusTip(tr("Clears all layers (makes the Stereogram nice and black again)"));
	ClearStereogram->setWhatsThis(tr("Clear Stereogram\n\nThis commands leavs colormap, depthmap and patternmap intact\nbut clears the stereogram."));
	connect(ClearStereogram, SIGNAL(activated()), this, SLOT(slotClearStereogram()));

	ClearDepthmap = new QAction(tr("Clear the depthmap"), tr("&Clear Depthmap"), 0, this);
	ClearDepthmap->setStatusTip(tr("Makes the depthmap nice and black again)"));
	connect(ClearDepthmap, SIGNAL(activated()), this, SLOT(slotClearDepthmap()));



	AcceptLayer =
		new QAction(tr("Lock Stereogram as Layer"), tr("&Lock Stereogram as layer"),
  QAccel::		stringToKey(tr("Ctrl+L")), this);
	AcceptLayer->setStatusTip(tr("Accept and lock the Stereogram as a layer, and make a new layer on top of it."));
	AcceptLayer->setWhatsThis(tr("Lock Layer\n\nThis commands will store and lock the current stereogram as a layer\nAll new renditions will be done on a new layer."));
	connect(AcceptLayer, SIGNAL(activated()), this,
			SLOT(slotAcceptLayer()));

	CopyLayer = new QAction(tr("Copy the current layer to the locked layer (discard old locked layer)"), tr("&Copy current layer"),
							0, this);
	CopyLayer->setStatusTip(tr("Copy the current layer to the locked."));
	CopyLayer->setWhatsThis(tr("Copy Layer\n\nThis commands copies the current layer to the locked layer"));
	connect(CopyLayer, SIGNAL(activated()), this, SLOT(slotCopyLayer()));

	CopyLayer2 = new QAction(tr("Copy the locked layer to the current layer (discard current layer)"), tr("&Copy locked layer"),
							 0, this);
	CopyLayer2->setStatusTip(tr("Copy the locked layer to the current."));
	CopyLayer2->setWhatsThis(tr("Copy Layer 2\n\nThis commands copies the locked layer to the current layer"));
	connect(CopyLayer2, SIGNAL(activated()), this, SLOT(slotCopyLayer2()));



	RejectLayer = new QAction(tr("Clear the current layer"), tr("&Clear (undo) current layer"),
  QAccel::				  stringToKey(tr("Ctrl+Z")), this);
	RejectLayer->setStatusTip(tr("Clear the current layer (undo all changes since the old layer were locked)."));
	RejectLayer->setWhatsThis(tr("Clear Layer\n\nThis commands clears the current layer and thus reject\nall rendering made since the old layer were locked"));
	connect(RejectLayer, SIGNAL(activated()), this,
			SLOT(slotRejectLayer()));


	SwapLayer = new QAction(tr("Swap the current layer and locked layer"), tr("&Swap the layers"),
  QAccel::				stringToKey(tr("Ctrl+X")), this);
	SwapLayer->setStatusTip(tr("Swaps the current layer and the locked layer with eachother."));
	SwapLayer->setWhatsThis(tr("Swap Layer\n\nThis commands exchange the current layer and the locked layer"));
	connect(SwapLayer, SIGNAL(activated()), this, SLOT(slotSwapLayer()));


	viewToolBar =
		new QAction(tr("Toolbar"), tr("Tool&bar"), 0, this, 0, true);
	viewToolBar->setStatusTip(tr("Enables/disables the toolbar"));
	viewToolBar->setWhatsThis(tr("Toolbar\n\nEnables/disables the toolbar"));
	connect(viewToolBar, SIGNAL(toggled(bool)), this,
			SLOT(slotViewToolBar(bool)));

	viewStatusBar =
		new QAction(tr("Statusbar"), tr("&Statusbar"), 0, this, 0, true);
	viewStatusBar->setStatusTip(tr("Enables/disables the statusbar"));
	viewStatusBar->
		setWhatsThis(tr("Statusbar\n\nEnables/disables the statusbar"));
	connect(viewStatusBar, SIGNAL(toggled(bool)), this,
			SLOT(slotViewStatusBar(bool)));

	viewDebug =
		new QAction(tr("Visual debug"), tr("&Visual debug"), 0, this, 0, true);
	viewDebug->setStatusTip(tr("Enables/disables debug info (useful for optimizing patternmapped renderer)"));
	viewDebug->
		setWhatsThis(tr("Debuginfo\n\nEnables/disables debug info. This means trace output and visual markers"));
	connect(viewDebug, SIGNAL(toggled(bool)), this,
			SLOT(slotViewDebug(bool)));



	loadDepthmap =
		new QAction(tr("Load Depthmap"), tr("&Load Depthmap..."),
  QAccel::		stringToKey(tr("F2")), this);
	loadDepthmap->
		setStatusTip(tr("Load the depthmap that describe the 3D shape"));
	loadDepthmap->
		setWhatsThis(tr
	  ("Load Depthmap\n\nLoad the depthmap that describe the 3D shape"));
	connect(loadDepthmap, SIGNAL(activated()), this,
			SLOT(slotLoadDepthmap()));


	saveDepthmap =
		new QAction(tr("Save Depthmap"), tr("&Save Depthmap..."),
					0, this);
	saveDepthmap->
		setStatusTip(tr("Saves the depthmap that describe the 3D shape"));
	saveDepthmap->setWhatsThis(tr("Save Depthmap\n\nSave the depthmap that describe the 3D shape"));
	connect(saveDepthmap, SIGNAL(activated()), this, SLOT(slotSaveDepthmap()));


	loadColormap =
		new QAction(tr("Load Colormap"), tr("&Load Colormap..."),
  QAccel::		stringToKey(tr("F3")), this);
	loadColormap->setStatusTip(tr("Load the colormap"));
	loadColormap->
		setWhatsThis(tr("Load Colormap\n\nHorizontal gradients looks awful!"));
	connect(loadColormap, SIGNAL(activated()), this,
			SLOT(slotLoadColormap()));

	loadPattern =
		new QAction(tr("Load Pattern"), tr("&Load Pattern..."),
  QAccel::		stringToKey(tr("F4")), this);
	loadPattern->setStatusTip(tr("Load the Pattern"));
	loadPattern->
		setWhatsThis(tr("Load Pattern\n\nBig patterns are nice too!"));
	connect(loadPattern, SIGNAL(activated()), this,
			SLOT(slotLoadPattern()));

	saveStereogram =
		new QAction(tr("Save Stereogram"), tr("&Save Stereogram..."), 0, this);
	saveStereogram->setStatusTip(tr("Save the Stereogram"));
	connect(saveStereogram, SIGNAL(activated()), this,
			SLOT(slotSaveStereogram()));


	loadStereogram =
		new QAction(tr("Load Stereogram"), tr("&Load Stereogram..."), 0, this);
	saveStereogram->setStatusTip(tr("Load an old Stereogram (into current layer)"));
	connect(loadStereogram, SIGNAL(activated()), this,
			SLOT(slotLoadStereogram()));


	addShape = new QAction(tr("Use Shapescript"), tr("&Shapescript..."),
						   0, this);
	addShape->setStatusTip(tr("Build depthmap using a script"));
	addShape->setWhatsThis(tr("Shapescript dialog\n\nThis is the main depthmap editing script tool."));
	connect(addShape, SIGNAL(activated()), this,
			SLOT(slotShapeScript()));


	helpAboutApp = new QAction(tr("About"), tr("&About..."),
  QAccel::				   stringToKey(tr("F1")), this);
	helpAboutApp->setStatusTip(tr("About the application"));
	helpAboutApp->setWhatsThis(tr("About\n\nAbout the application"));
	connect(helpAboutApp, SIGNAL(activated()), this,
		SLOT(slotHelpAbout()));

}

void
 KrosseyeApp::initMenuBar()
{
	// menuBar entry fileMenu
	//
	fileMenu = new QPopupMenu();
	fileNew->addTo(fileMenu);
	fileMenu->insertSeparator();
	saveStereogram->addTo(fileMenu);
	loadStereogram->addTo(fileMenu);
	ClearStereogram->addTo(fileMenu);
	fileMenu->insertSeparator();
	saveDepthmap->addTo(fileMenu);
	loadDepthmap->addTo(fileMenu);
	ClearDepthmap->addTo(fileMenu);
	fileMenu->insertSeparator();
	loadColormap->addTo(fileMenu);
	fileMenu->insertSeparator();
	loadPattern->addTo(fileMenu);
	fileMenu->insertSeparator();
	fileQuit->addTo(fileMenu);

	//  filePrint->addTo(fileMenu);
	layerMenu = new QPopupMenu();
	viewMenu = new QPopupMenu();
	toolMenu = new QPopupMenu();
	helpMenu = new QPopupMenu();

	AcceptLayer->addTo(layerMenu);
	RejectLayer->addTo(layerMenu);
	layerMenu->insertSeparator();
	CopyLayer->addTo(layerMenu);
	CopyLayer2->addTo(layerMenu);
	SwapLayer->addTo(layerMenu);
	layerMenu->insertSeparator();


	viewMenu->setCheckable(true);
	viewStatusBar->addTo(viewMenu);
	viewDebug->addTo(viewMenu);
	//viewToolBar->addTo (viewMenu);
	viewMenu->insertSeparator();
	addShape->addTo(viewMenu);

	helpAboutApp->addTo(helpMenu);

	menuBar()->insertItem(tr("&File"), fileMenu);
	menuBar()->insertItem(tr("&Layer"), layerMenu);
	menuBar()->insertItem(tr("&View"), viewMenu);
	//menuBar ()->insertItem (tr ("&Tools"), toolMenu);
	menuBar()->insertItem(tr("&Help"), helpMenu);

}

void
 KrosseyeApp::initToolBar()
{
	fileToolbar = new QToolBar(this, "file operations");
	fileNew->addTo(fileToolbar);
	fileToolbar->addSeparator();
  QWhatsThis::whatsThisButton(fileToolbar);

}

void
 KrosseyeApp::initStatusBar()
{
	statusBar()->message(tr("Ready."), 2000);
}

void
 KrosseyeApp::initDoc()
{
	doc = new KrosseyeDoc();
}

void
 KrosseyeApp::initView()
{
	view = new KrosseyeView(this, doc);
	setCentralWidget(view);
}

bool
KrosseyeApp::queryExit()
{
	int exit = QMessageBox::information(this, tr("Quit..."),
									  tr("Do your really want to quit?"),
										QMessageBox::Ok,
										QMessageBox::Cancel);


	return (exit == 1);
}

void
 KrosseyeApp::slotFileNew()
{

	statusBar()->message(tr("Creating new file..."));

	dlgNew *dlg = new dlgNew(this, "dlg");
	dlg->SetData(doc);
	if (dlg->exec())
	{
		dlg->GetData(doc);
		doc->newDoc();
	}
	delete dlg;

	statusBar()->message(tr("Ready."));
}


void
 KrosseyeApp::slotFilePrint()
{
	statusBar()->message(tr("Printing..."));
	QPrinter printer;
	if (printer.setup(this))
	{
		QPainter painter;
		painter.begin(&printer);

		painter.end();
	};

	statusBar()->message(tr("Ready."));
}

void
 KrosseyeApp::slotFileQuit()
{
	statusBar()->message(tr("Exiting application..."));
	if (doc->isModified())
	{
		if (queryExit())
		{
			qApp->quit();
		}
		else
		{

		};
	}
	else
	{
		qApp->quit();
	};

	statusBar()->message(tr("Ready."));
}

void
 KrosseyeApp::slotViewToolBar(bool toggle)
{
	statusBar()->message(tr("Toggle toolbar..."));

	if (toggle == false)
	{
		fileToolbar->hide();
	}
	else
	{
		fileToolbar->show();
	};

	statusBar()->message(tr("Ready."));
}

void
 KrosseyeApp::slotViewStatusBar(bool toggle)
{
	statusBar()->message(tr("Toggle statusbar..."));
	if (toggle == false)
	{
		statusBar()->hide();
	}
	else
	{
		statusBar()->show();
	}

	statusBar()->message(tr("Ready."));
}

void
 KrosseyeApp::slotViewDebug(bool toggle)
{
	if(doc)
		doc->viewdebug=toggle;
}

void
 KrosseyeApp::slotHelpAbout()
{
	//  QMessageBox::about(this,tr("About..."),
	//                      tr("Krosseye\nVersion " VERSION "\n(c) 2001 by Tersevs") );
	//
	dlgAbout *dlg = new dlgAbout(this, "dlg");
	dlg->exec();
	delete dlg;
}

/** No descriptions */
void
 KrosseyeApp::slotLoadDepthmap()
{
	view->ConfigureDocument();
	statusBar()->message(tr("Load Depthmap..."));
	QString s(QFileDialog::
			  getOpenFileName(QString::null, SUPPORTED_IMG_FORMATS,
							  this));

	statusBar()->message(tr("Ready."));

	if (s.isEmpty())
		return;

	doc->loadDepthmap(s);

}

/** No descriptions */
void
 KrosseyeApp::slotLoadColormap()
{
	view->ConfigureDocument();
	statusBar()->message(tr("Load Color..."));
	QString s(QFileDialog::
			  getOpenFileName(QString::null, SUPPORTED_IMG_FORMATS,
							  this));

	statusBar()->message(tr("Ready."));

	if (s.isEmpty())
		return;

	doc->loadColormap(s);

}

/** No descriptions */
void
 KrosseyeApp::slotLoadPattern()
{
	view->ConfigureDocument();
	statusBar()->message(tr("Load Pattern..."));
	QString s(QFileDialog::
			  getOpenFileName(QString::null, SUPPORTED_IMG_FORMATS,
							  this));

	statusBar()->message(tr("Ready."));

	if (s.isEmpty())
		return;

	doc->loadPattern(s);


}


/** No descriptions */
void
 KrosseyeApp::slotLoadStereogram()
{
	view->ConfigureDocument();
	statusBar()->message(tr("Load Stereogram..."));
	QString s(QFileDialog::
			  getOpenFileName(QString::null, SUPPORTED_IMG_FORMATS,
							  this));

	statusBar()->message(tr("Ready."));

	if (s.isEmpty())
		return;

	doc->loadStereogram(s);


}


/** No descriptions */
void
 KrosseyeApp::slotSaveStereogram()
{
	view->ConfigureDocument();
	statusBar()->message(tr("Save Stereogram..."));
	QString s(QFileDialog::getSaveFileName(QString::null, "*.png", this));

	statusBar()->message(tr("Ready."));

	if (s.isEmpty())
		return;

	doc->saveStereogram(s);
}

void
 KrosseyeApp::slotSaveDepthmap()
{
	view->ConfigureDocument();
	statusBar()->message(tr("Save Depthmap..."));
	QString s(QFileDialog::getSaveFileName(QString::null, "*.png", this));

	statusBar()->message(tr("Ready."));

	if (s.isEmpty())
		return;

	doc->saveDepthmap(s);
}


/** No descriptions */
void
 KrosseyeApp::slotProcessing(int percent_done)
{
	char buffer[50];
	if (percent_done < 100)
	{
		sprintf(buffer, tr("Processing - %d percent done..."), percent_done);
		statusBar()->message(buffer);
	}
	else
	{
		statusBar()->message(tr("Ready."));
	}
}

void
 KrosseyeApp::slotMousePos(int x, int y)
{
	char buffer[50];
	if (x < 0 || x >= doc->width || y < 0 || y >= doc->heigth)
	{
		statusBar()->message(tr("Ready."));
	}
	else
	{
		sprintf(buffer, tr("Mouse @ ( %d , %d ) Rawdepth = %d"), x - doc->getWidth() / 2, y - doc->getHeigth() / 2, doc->GetRawDepth(x, y));
		statusBar()->message(buffer);
	}
}

void
 KrosseyeApp::slotClearStereogram()
{
	view->ConfigureDocument();
	doc->clearStereogram();
}

void
 KrosseyeApp::slotClearDepthmap()
{
	view->ConfigureDocument();
	doc->clearDepthmap();
}


void
 KrosseyeApp::slotAcceptLayer()
{
	view->ConfigureDocument();
	doc->AcceptLayer();
}

void
 KrosseyeApp::slotRejectLayer()
{
	view->ConfigureDocument();
	view->ConfigureDocument();
	doc->RejectLayer();
}

void
 KrosseyeApp::slotCopyLayer()
{
	view->ConfigureDocument();
	doc->CopyCurrentLayer();
}

void
 KrosseyeApp::slotCopyLayer2()
{
	view->ConfigureDocument();
	doc->CopyLockedLayer();
}


void
 KrosseyeApp::slotSwapLayer()
{
	view->ConfigureDocument();
	doc->SwapLayer();
}

void
 KrosseyeApp::slotShapeScript()
{
	ShapeScript->show();
	statusBar()->message(tr("Ready."));
}


void KrosseyeApp::initDialogs()
{
	ShapeScript = new dlgShapeScript(doc, this, "dlg");
}
