/***************************************************************************
                          krosseye.h  -  description
                             -------------------
    begin                : Sun Jun 10 19:09:16 CEST 2001
    copyright            : (C) 2001 by Tersevs
    email                : tersevs@home.se
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KROSSEYE_H
# define KROSSEYE_H

// include files for QT
# include <qapp.h>
# include <qmainwindow.h>
# include <qaction.h>
# include <qmenubar.h>
# include <qpopupmenu.h>
# include <qtoolbar.h>
# include <qtoolbutton.h>
# include <qstatusbar.h>
# include <qwhatsthis.h>
# include <qstring.h>
# include <qpixmap.h>
# include <qmsgbox.h>
# include <qfiledialog.h>
# include <qprinter.h>
# include <qpainter.h>
# include <qtimer.h>
# include <qpixmap.h>

// application specific includes
# include "krosseyeview.h"
# include "krosseyedoc.h"
# include "dlgShapeScript.h"

/**
  * This Class is the base class for your application. It sets up the main
  * window and providing a menubar, toolbar
  * and statusbar. For the main view, an instance of class KrosseyeView is
  * created which creates your view.
  */
class KrosseyeApp : public QMainWindow
{
   Q_OBJECT

 public:
    /** construtor */
   KrosseyeApp();
    /** destructor */
   ~KrosseyeApp();
    /** initializes all QActions of the application */
   void initActions();
    /** initMenuBar creates the menu_bar and inserts the menuitems */
   void initMenuBar();
    /** this creates the toolbars. Change the toobar look and add new toolbars in this
     * function */
   void initToolBar();
    /** setup the statusbar */
   void initStatusBar();
    /** setup the document*/
   void initDoc();
    /** setup the mainview*/
   void initView();
   /** setup dialogs */
   void initDialogs();
   /** overloaded for Message box on last window exit */
   bool queryExit();

 public slots:


    /** generate a new document in the actual view */
   void slotFileNew();
    /** print the actual file */
   void slotFilePrint();
    /** exits the application */
   void slotFileQuit();
    /** toggle the toolbar*/
   void slotViewToolBar(bool toggle);
    /** toggle the statusbar*/
   void slotViewStatusBar(bool toggle);
   /** toggle debug output*/
   void slotViewDebug(bool toggle);

    /** shows an about dlg*/
   void slotHelpAbout();
    /** No descriptions */
   void slotLoadDepthmap();
   /** No descriptions */
   void slotSaveDepthmap();
    /** No descriptions */
   void slotLoadColormap();
    /** No descriptions */
   void slotLoadPattern();
    /** No descriptions */
   void slotSaveStereogram();
   /** No descriptions */
   void slotLoadStereogram();

  	/** No descriptions */
   void slotProcessing(int percent_done);
  	/** No descriptions */
   void slotMousePos(int x, int y);


   void slotClearStereogram();
   void slotClearDepthmap();
   void slotAcceptLayer();
   void slotRejectLayer();
   void slotCopyLayer();
   void slotCopyLayer2();
   void slotSwapLayer();
   void slotShapeScript();


 
 private:

    /** view is the main widget which represents your working area. The View
     * class should handle all events of the view widget.  It is kept empty so
     * you can create your view according to your application's needs by
     * changing the view class.
     */
   KrosseyeView *view;
    /** doc represents your actual document and is created only once. It keeps
     * information such as filename and does the serialization of your files.
     */
   KrosseyeDoc *doc;
	
   /* Depthmap add shape dialog */
   dlgShapeScript *ShapeScript;

    /** file_menu contains all items of the menubar entry "File" */
   QPopupMenu *fileMenu;
    /** edit_menu contains all items of the menubar entry "Layer" */
   QPopupMenu *layerMenu;
    /** view_menu contains all items of the menubar entry "View" */
   QPopupMenu *viewMenu;
	/** tool_menu contains all items of the menubas entry "Data" */
   QPopupMenu *toolMenu;
    /** view_menu contains all items of the menubar entry "Help" */
   QPopupMenu *helpMenu;
    /** the main toolbar */
   QToolBar *fileToolbar;
    /** actions for the application initialized in initActions() and used to en/disable them
      * according to your needs during the program */
   QAction *fileNew;
   QAction *filePrint;
   QAction *fileQuit;

		/* Edit meny */
   QAction *ClearStereogram;
   QAction *ClearDepthmap;

   QAction *AcceptLayer;
   QAction *RejectLayer;
   QAction *CopyLayer;
   QAction *CopyLayer2;
   QAction *SwapLayer;

   QAction *viewToolBar;
   QAction *viewStatusBar;
   QAction *viewDebug;

   QAction *loadPattern;
   QAction *loadDepthmap;
   QAction *saveDepthmap;
   QAction *loadColormap;
   QAction *saveStereogram;
   QAction *loadStereogram;

   QAction *addShape;
   QAction *helpAboutApp;

   QPixmap *pixIcon;

};
#endif

