/***************************************************************************
                          krosseyedoc.h  -  description
                             -------------------
    begin                : Sun Jun 10 19:09:16 CEST 2001
    copyright            : (C) 2001 by Tersevs
    email                : tersevs@home.se
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef KROSSEYEDOC_H
# define KROSSEYEDOC_H

// include files for QT
# include <qobject.h>
# include <qimage.h>
# include <qtimer.h>

// application specific includes
//
/**
  * the Document Class
  */

class KrosseyeDoc : public QObject
{
   Q_OBJECT

 public:
   enum rendmeth { REND_RANDOMCOLORS, REND_COLORMAPPED, REND_PATTERNMAPPED, REND_SCATTERED_DOTS, REND_PATTERNDOTTED, REND_GOGGLES, REND_PUNCHED };
   enum lymrmeth { LYMR_BRIGHTEST, LYMR_LATEST, LYMR_AVERAGE, LYMR_TRANS };
   enum mousefunc { MOUSE_NONE, MOUSE_RAISE, MOUSE_LOWER, MOUSE_SOFTEN, MOUSE_SPREAD, MOUSE_BRIDGE1, MOUSE_BRIDGE2, MOUSE_DOT };
   enum depdmode { DEPD_NORMAL, DEPD_RAISE, DEPD_LOWER, DEPD_CUTBELOW };
   enum nullmode { NULM_IGNORE, NULM_BRIDGE, NULM_OL_ALIGN, NULM_OL_MIRRORED, NULM_OL_FAR };

   /** Constructs and initializes */
    KrosseyeDoc();
    /** Destructor */
    ~KrosseyeDoc();

	/** Gets width */
	int getWidth(void);
	/** Gets heigth */
	int getHeigth(void);
	/** Set width */
	void setWidth(int w);
	/** Set heigth */
	void setHeigth(int h);


	/** Returns info about if the document has been modified since last save. Not used now */
    bool isModified() const;
  	/** Starts a new steregoram, with a new width and heigth */
    void newDoc();
    /** Make the stereogram all black */
    void clearStereogram(void);
    /** Make the depthmap all black */
    void clearDepthmap(void);
    /** makes sure the view is redrawn */
	void ForceDocumentChanged();

	/*** File save/load methods */
	
	/** load Depthmap from a file */
    bool loadDepthmap(const QString & str);
	/** save Depthmap to a file */
    bool saveDepthmap(const QString & str);
  	/** Load colormap from a file */
    bool loadColormap(const QString & str);
   	/** Load pattern from a file */
    bool loadPattern(const QString & str);
  	/** Save the steregoram to a file */
    bool saveStereogram(const QString & str);
  	/** Load the steregoram from a file into current layer */
    bool loadStereogram(const QString & str);

	/*** Depthmap editing methods ***/

	/** Edit the depthmap by raising an area. The mouse_brush_size and mouse_brush_pow decides how much... */ 
	void raise_depthmap_by_mouseclick(int x, int y);
	/** Edit the depthmap by lowering an area. The mouse_brush_size and mouse_brush_pow decides how much... */ 
	void lower_depthmap_by_mouseclick(int x, int y);
	/** Edit the depthmap by smoothing an area. The mouse_brush_size and mouse_brush_pow decides how much... */ 
	void soften_depthmap_by_mouseclick(int x, int y, int pow=-1);
	/** Edit the depthmap by 'spreading' an area. The mouse_brush_size and mouse_brush_pow decides how much... */ 
	void spread_depthmap_by_mouseclick(int x, int y);
	/** Edit the depthmap by bridging an area. The mouse_brush_size and mouse_brush_pow decides how much... */ 
	void bridge1_depthmap_by_mouseclick(int x1, int y1, int x2, int y2);
	/** Edit the depthmap by bridging an area. The mouse_brush_size and mouse_brush_pow decides how much... */ 
	void bridge2_depthmap_by_mouseclick(int x1, int y1, int x2, int y2);
	/** Edit the depthmap */
	void dot_depthmap_by_mouseclick(int x, int y);

    /** Read depth from depthmap */
    int GetRawDepth(int x, int y);
    /** Read depth from depthmap and scales it */
    float GetDepth(int x, int y);
    /** Read depth from depthmap */
    void SetRawDepth(int x, int y, int depth);
	/** Draw a dot on the depthmap */
	void drawDepthmap_dot(int x, int y, int z, int size, int arg=0);
	/** Draw a line on the depthmap */
	void drawDepthmap_line(int ax, int ay, int az, int bx, int by, int bz, int size, int pow);
	/** Draw a flatline on the depthmap */
	void drawDepthmap_flatline(int ax, int ay, int az, int bx, int by, int bz, int size);
	/** Soften area on the depthmap */
	void drawDepthmap_soften(int cx, int cy, int size, int pow);	
	/** Draw triangle on the depthmap */
	void drawDepthmap_triangle(int ax, int ay, int az, int bx, int by, int bz, int cx, int cy, int cz);	
	/** Draw cone on the depthmap */
	void drawDepthmap_cone(int cx, int cy, int cz, int bz, int size);	
	/** Draw sphere on the depthmap */
	void drawDepthmap_sphere(int cx, int cy, int cz, int width, int height, int basez, int arg);
	/** Draw disc on the depthmap */
	void drawDepthmap_disc(int cx, int cy, int cz, int rad, int arg1, int arg2);
	/** Copy and transform part of depthmap */
	void drawDepthmap_copy(int sx, int sy, int sw, int sh, int dx, int dy,  int argx, int argy, int argz );
	/** raise (or lower) part of depthmap */
	void drawDepthmap_raise(int cx, int cy, int rad, int dist, int pow );	
	/** Load and merge depthmap from file */
	void drawDepthmap_load(const QString & str, int cx=0, int cy=0, int cz=0, int sizex=320, int sizey=200, int arg=0);	

	

	/*** Rendering methods ***/

    /** Sets up the render statemachine for a full rendering and then calls render_start() to begin */
    void render_by_buttonpress(void);
    /** Sets up the render statemachine for a parial rendering - in neccesary - and start it */
    void render_by_mouseclick(int x, int y, int x2=-100000, int y2=-100000);
    /** Sets the desired render method. Use this before starting a rendering */
    void setRenderMethod(enum rendmeth new_method);
    /** Indicates if the rendering state-machine is running */
    bool isRendering(void);
    /** Internal function to start the rendering state-machine */
    void renderer_start(void);
    /** Clen up / finish a rendering */
    void renderer_stop(void);
    /** This function renders a screen line and returns (uses render_dot) */
    bool renderer_tick(void ); 

	/*** Stereogram layer methods ***/
	
	/** Update stereogram from new and old layer */
	void UpdateStereogram();
	/** Save stereogram as old layer and clear the new layer */
	void AcceptLayer();
	/** Clear the new layer */
	void RejectLayer();
	/** Copy the new layer to the old*/
	void CopyCurrentLayer();
	/** Copy the locked layer to the onew*/
	void CopyLockedLayer();
	/** Swap the new layer and the old*/
	void SwapLayer();

	/*** Variables ***/

	/** Width of the stereogram */
    int	width;
	/** height of the stereogram (yeah, misspelled) */
    int heigth;
	/** Minimum offset from the screen plane */
    int mindepth;
	/** How far away from the screen plane the shape can be */
    int	maxdepth;
	/** Antialias strength (1-) */
	int antialias;
	/** Invert depthmap */
    int invert_depth;
	/** Horizontal depthmap smoothing */
    int depthmap_smoothing;
	/** Is black pixels in patterns top be trated as (completly) transparent */
    bool black_is_trans;
	/* Show debug info */
	bool viewdebug;
	/** Rotate dots? */
    bool rotate_dots;
	/** Update screen when rendering */
	bool redraw_screen_often;	
	/** Transparancy for avg layer merginf */
	float layer_avg_trans;
	/** Layer mergng method */
	enum lymrmeth layer_merging_method;
	/** What a mouse click does */
	enum mousefunc mouse_function;
	/** Mouse brush size for depthmap edit */
	int mouse_brush_size;
	/** Mouse brush strength for depthmap edit */
	int mouse_brush_pow;
	/** depthmap drawing mode  0=overvrite 1=raiseonly 2=loweronly */
    enum depdmode depthmap_drawing_mode;
	/** Method for nullzone handelling */
    enum nullmode nullzone_mode;
	/**	How much deptherror we can allow */
    int tolerance;
  	/** No descriptions */
    int dotsize;
	/** Horizontal color dissipation */
    float	color_dilution;
	/** Horizontal color scattering */
    int	  color_scatter;
   	/** Horizontal origin: When rendering a patternmap, the 'startpixels' will all originate from this */
    int	horiz_orgin_perc;
	/** Vertical offset */
	int vert_orgin_perc;	
	/** Handle in center? */
	bool horiz_handle_centered;
	/** Render method */
    enum rendmeth current_method;
	/** Color of left goggle */
	QColor goggle_left_color;
	/** Color of left goggle */
	QColor goggle_right_color;

    /*** Keep track of the rendering state machine ***/
	class renderstate_class
	{
		public:
		void newdoc(int heigth);

		void setup(int startline, int stopline);
		void start(void);
		void stop(void);
		
		bool IsRendering(void);
		bool RenderingLastLine(void);

		bool TimeToRefreshScreen(void);

		int  GetLine(void);
		void StepLine(int inc);
		int  GetPercentDone(void);

		protected:
		int first_row_to_render;
		int last_row_to_render;
	    int ycurrent;
		int update_counter;
		int aheigth;

	} renderstate;

	/*** Horizontal origin manager ***/
	class hotizorg_class
	{
		public: 
			hotizorg_class();	
			~hotizorg_class();	

			void newdoc(int w,int h);
			int get(int y);
			void set(int y, int o);

		protected:
			int *horiz_orgin;
			int heigth;
			int width;
	} horizorgin;


	/*** Keep track of what lines that has been updated since last UpdateStereogram() call ***/

	class update_class
	{
		public:		
		void make_all_clean(void);
		void make_all_dirty(void);
		
		void make_dirty(int x, int y);

		int startline;
		int stopline;
		int startrow;
		int stoprow;

	} update;


	

   QImage *StereogramImage, *DepthmapImage, *ColormapImage, *PatternImage;
   QImage *NewLayerImage, *OldLayerImage;

public slots:	
  /** Tick drives the rendering statemachine */
   void slotTick();

signals:
   /** Signal that the document has been change so that the view must be uipdated */
   void documentChanged();
   /** Canbe used to show a nice progress report for the rendering */
   void signalPercentDone(int percent);

protected:
  
   /*** Protected helper functions ***/

   /** Indicates if the document has been modified. Not used right now */
   bool modified;
   /** The main rendering magic happens here. Plotts a row (not a full screen line) of dots */
   bool render_dot(	int *depth, uint *color, int *filled,
		int xstart, int y, bool allow_incomplete_rows=false, int mark_dots=1);
    /** This function renders a screen line and returns (uses render_dot) */
    bool render_row(int y); 

   /** This macro prepares databuffers for a line that is to be rendered.*/
   void Prepare_row_data(int y, int *depth, int *filled , unsigned int *color);
   /** Calculates what X-position (index in color buffer) to fetch color from */
   int ScatterX(int x);
   /** Draws a dot on the stereogram */
   void DrawDot(int x, int y, QRgb col, int rotation=0);
   /** Draws a pixel on the stereogram */
   void DrawPixel(int x, int y, QRgb col);
   /** Calculate the average raw depth around a pixel */
   int calc_rawdm_area_avg(int cx, int cy, int area);

   /** Read color from colormap (and does scattering) */
   QRgb GetColor(int x,int y);

   
   /** Drives the rendering state machine */
   QTimer *timer;

};

#endif

