

/***************************************************************************
                          krosseyeview.cpp  -  description
                             -------------------
    begin                : Sun Jun 10 19:09:16 CEST 2001
    copyright            : (C) 2001 by Tersevs
    email                : tersevs@home.se
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "krosseyeview.h"
#include <stdio.h>
#include <stdlib.h>
#include <qlineedit.h>
#include <qslider.h>
#include <qmessagebox.h>
#include <qcheckbox.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qgroupbox.h>
#include <qbuttongroup.h>
#include <qcolordialog.h>
#include <qcolor.h>

#define MET_RANDOMCOLORS					0
#define MET_COLORMAPPED						1
#define MET_PATTERNMAPPED					2
#define MET_PATTERNDOTTED                   3
#define MET_SCATTERED_DOTS					4
#define MET_PUNCHED							5
#define MET_GOGGLES							6


#define LMR_BRIGHTEST						0
#define LMR_LATEST							1
#define LMR_AVERAGE							2
#define LMR_TRANS							3

#define MOU_NONE							0
#define MOU_RAISE							1
#define MOU_LOWER							2
#define MOU_SOFTEN							3
#define MOU_SPREAD							4
#define MOU_BRIDGE1							5
#define MOU_BRIDGE2							6
#define MOU_DOT								7

#define TAB_STEREOGRAM_PAGE_INDEX			0
#define TAB_DEPTHMAP_PAGE_INDEX				1
#define TAB_COLORMAP_PAGE_INDEX				2
#define TAB_PATTERN_PAGE_INDEX				3


#define NUM_IGNORE							0
#define NUM_BRIDGE							1
#define NUM_OL_ALIGN						2
#define NUM_OL_MIRRORED						3
#define NUM_OL_FAR							4

KrosseyeView:: KrosseyeView(QWidget * parent, KrosseyeDoc * doc):krosseyeview_ui(parent)
{
/** connect doc with the view*/
	connect(doc, SIGNAL(documentChanged()), this,
			SLOT(slotDocumentChanged()));

	connect(doc, SIGNAL(signalPercentDone(int)), this,
			SLOT(slotProcessing(int)));

	StereogramPixmap = NULL;
	DepthmapPixmap = NULL;
	ColormapPixmap = NULL;
	PatternPixmap = NULL;
	width = 0;
	heigth = 0;
  mouse_pressed = QMouseEvent::NoButton;

	/* Stereogram tab */
	StereogramScrollView =
		new QScrollView(StereogramPage, "StereogramScrollView");
  StereogramScrollView->setResizePolicy(QScrollView::AutoOne);
	StereogramScrollView->viewport()->setBackgroundMode(PaletteBackground);
	StereogramLabel = new MLabel(StereogramScrollView, "StereogramLabel");
	StereogramScrollView->addChild(StereogramLabel);
	StereogramPageLayout->addWidget(StereogramScrollView, 0, 0);
	connect(StereogramLabel, SIGNAL(signalMousePressEvent(QMouseEvent)), this,
			SLOT(slotMousePressEvent(QMouseEvent)));
	connect(StereogramLabel, SIGNAL(signalMouseReleaseEvent(QMouseEvent)), this,
			SLOT(slotMouseReleaseEvent(QMouseEvent)));
	connect(StereogramLabel, SIGNAL(signalMouseMoveEvent(QMouseEvent)), this,
			SLOT(slotMouseMoveEvent(QMouseEvent)));

	/* Depthmap tab */

	DepthmapScrollView = new QScrollView(DepthmapPage, "DepthmapScrollView");
  DepthmapScrollView->setResizePolicy(QScrollView::AutoOne);
	DepthmapScrollView->viewport()->setBackgroundMode(PaletteBackground);
	DepthmapLabel = new MLabel(DepthmapScrollView, "DepthmapLabel");
	DepthmapScrollView->addChild(DepthmapLabel);
	DepthmapPageLayout->addWidget(DepthmapScrollView, 0, 0);
	connect(DepthmapLabel, SIGNAL(signalMousePressEvent(QMouseEvent)), this,
			SLOT(slotMousePressEvent(QMouseEvent)));
	connect(DepthmapLabel, SIGNAL(signalMouseReleaseEvent(QMouseEvent)), this,
			SLOT(slotMouseReleaseEvent(QMouseEvent)));
	connect(DepthmapLabel, SIGNAL(signalMouseMoveEvent(QMouseEvent)), this,
			SLOT(slotMouseMoveEvent(QMouseEvent)));


	/* Colormap tab */

	ColormapScrollView = new QScrollView(ColormapPage, "ColormapScrollView");
  ColormapScrollView->setResizePolicy(QScrollView::AutoOne);
	ColormapScrollView->viewport()->setBackgroundMode(PaletteBackground);
	ColormapLabel = new MLabel(ColormapScrollView, "ColormapLabel");
	ColormapScrollView->addChild(ColormapLabel);
	ColormapPageLayout->addWidget(ColormapScrollView, 0, 0);
	connect(ColormapLabel, SIGNAL(signalMousePressEvent(QMouseEvent)), this,
			SLOT(slotMousePressEvent(QMouseEvent)));
	connect(ColormapLabel, SIGNAL(signalMouseReleaseEvent(QMouseEvent)), this,
			SLOT(slotMouseReleaseEvent(QMouseEvent)));
	connect(ColormapLabel, SIGNAL(signalMouseMoveEvent(QMouseEvent)), this,
			SLOT(slotMouseMoveEvent(QMouseEvent)));

	/* Pattern tab */

	PatternScrollView = new QScrollView(PatternPage, "PatternScrollView");
  PatternScrollView->setResizePolicy(QScrollView::AutoOne);
	PatternScrollView->viewport()->setBackgroundMode(PaletteBackground);
	PatternLabel = new MLabel(PatternScrollView, "PatternLabel");
	PatternScrollView->addChild(PatternLabel);
	PatternPageLayout->addWidget(PatternScrollView, 0, 0);
	connect(PatternLabel, SIGNAL(signalMousePressEvent(QMouseEvent)), this,
			SLOT(slotMousePressEvent(QMouseEvent)));
	connect(PatternLabel, SIGNAL(signalMouseReleaseEvent(QMouseEvent)), this,
			SLOT(slotMouseReleaseEvent(QMouseEvent)));
	connect(PatternLabel, SIGNAL(signalMouseMoveEvent(QMouseEvent)), this,
			SLOT(slotMouseMoveEvent(QMouseEvent)));

	Doc = doc;

	/* Refresh everything */
	slotDocumentChanged();

}

KrosseyeView::~KrosseyeView()
{
}


void
 KrosseyeView::slotDocumentChanged()
{
	char buffer[20];

	/* Misc */
//  chbRedrawOften->setChecked( Doc->redraw_screen_often);

	/* Depthmaop offset and maxdepth */
	sprintf(buffer, "%d", Doc->mindepth);
	txtMindepth->setText(buffer);
	sprintf(buffer, "%d", Doc->maxdepth);
	txtMaxdepth->setText(buffer);
	cbInverted->setChecked(Doc->invert_depth);
	Smoothing->setValue(Doc->depthmap_smoothing);

	/* Colormap scatter and dilutions */
	Scatter->setValue(Doc->color_scatter);
	Dilute->setValue((int) (Doc->color_dilution * 100));

	/*dotsize */
	slDotsize->setValue(Doc->dotsize);

	/* Mouse option */
	cbMouse->setCurrentItem(Doc->mouse_function);	/* MAKE SWITCH LATER */
	slBrushSize->setValue((int) (Doc->mouse_brush_size));
	slBrushPow->setValue((int) (Doc->mouse_brush_pow));

	/* Pattern options */
	cbBlackIsTrans->setChecked(Doc->black_is_trans);
	chbRotateDots->setChecked(Doc->rotate_dots);
	chbHCent->setChecked(Doc->horiz_handle_centered);

	/* Layer options */
	cbLayerMerge->setCurrentItem(Doc->layer_merging_method);	/* MAKE SWITCH LATER */

	cbNullzones->setCurrentItem(Doc->nullzone_mode);



	HOrgin->setValue(Doc->horiz_orgin_perc);
	VOrgin->setValue(Doc->vert_orgin_perc);

	slotConfig();
}

/** No descriptions */
void
 KrosseyeView::slotConfig()
{
	bool resized;
	QPixmap *p;


	if ((width == Doc->getWidth()) && (heigth == Doc->getHeigth()))
		resized = false;
	else
		resized = true;

/** Update the pixmaps etc */
	switch (TabWidget1->currentPageIndex())
	{
		case TAB_STEREOGRAM_PAGE_INDEX:
			PaintGroupBox->setEnabled(false);
			p = 0;
			if (resized)
			{
				p = StereogramPixmap;
				StereogramPixmap = new QPixmap(Doc->getWidth(), Doc->getHeigth());
				if (!StereogramPixmap)
				{
					printf("Out of memory?\n");
					exit(-1);
				}
			}
			StereogramPixmap->convertFromImage(*Doc->StereogramImage);
			StereogramLabel->setPixmap(*StereogramPixmap);
			if (p)
			{
				delete(p);
			}
			break;

		case TAB_DEPTHMAP_PAGE_INDEX:
			PaintGroupBox->setEnabled(true);
			p = 0;
			if (resized)
			{
				p = DepthmapPixmap;
				DepthmapPixmap = new QPixmap(Doc->getWidth(), Doc->getHeigth());
				if (!DepthmapPixmap)
				{
					printf("Out of memory?\n");
					exit(-1);
				}
			}
			DepthmapPixmap->convertFromImage(*Doc->DepthmapImage);
			DepthmapLabel->setPixmap(*DepthmapPixmap);
			if (p)
			{
				delete(p);
			}
			break;

		case TAB_COLORMAP_PAGE_INDEX:
			PaintGroupBox->setEnabled(false);
			p = 0;
			if (resized)
			{
				p = ColormapPixmap;
				ColormapPixmap = new QPixmap(Doc->getWidth(), Doc->getHeigth());
				if (!ColormapPixmap)
				{
					printf("Out of memory?\n");
					exit(-1);
				}
			}
			ColormapPixmap->convertFromImage(*Doc->ColormapImage);
			ColormapLabel->setPixmap(*ColormapPixmap);
			if (p)
			{
				delete(p);
			}
			break;

		case TAB_PATTERN_PAGE_INDEX:
			PaintGroupBox->setEnabled(false);
			p = 0;
			if (resized)
			{
				p = PatternPixmap;
				PatternPixmap = new QPixmap(Doc->getWidth(), Doc->getHeigth());
				if (!PatternPixmap)
				{
					printf("Out of memory?\n");
					exit(-1);
				}
			}
			PatternPixmap->convertFromImage(*Doc->PatternImage);
			PatternLabel->setPixmap(*PatternPixmap);
			if (p)
			{
				delete(p);
			}
			break;
	}


//   printf("slotConfig\n");

	if (cbMouse->currentItem() == 0)
	{
		mousetext1->setEnabled(false);
		mousetext2->setEnabled(false);
		slBrushSize->setEnabled(false);
		slBrushPow->setEnabled(false);
	}
	else
	{
		mousetext1->setEnabled(true);
		mousetext2->setEnabled(true);
		slBrushSize->setEnabled(true);
		slBrushPow->setEnabled(true);

	}


	/* method */

	switch (MethodComboBox->currentItem())
	{
		case MET_RANDOMCOLORS:
			TabWidget1->setTabEnabled(ColormapPage, false);
			TabWidget1->setTabEnabled(PatternPage, false);
			ColormapGroupBox->setEnabled(false);
			PatternmapGroupBox->setEnabled(false);
			GogglesGroupBox->setEnabled(false);
			chbRotateDots->setEnabled(false);
			//slDotsize->setEnabled(true);
			break;

		case MET_GOGGLES:
			TabWidget1->setTabEnabled(ColormapPage, false);
			TabWidget1->setTabEnabled(PatternPage, false);
			ColormapGroupBox->setEnabled(false);
			PatternmapGroupBox->setEnabled(false);
			GogglesGroupBox->setEnabled(true);
			chbRotateDots->setEnabled(false);
			//slDotsize->setEnabled(true);
			break;

		case MET_COLORMAPPED:
		case MET_SCATTERED_DOTS:		
			TabWidget1->setTabEnabled(ColormapPage, true);
			TabWidget1->setTabEnabled(PatternPage, false);
			ColormapGroupBox->setEnabled(true);
			PatternmapGroupBox->setEnabled(false);
			GogglesGroupBox->setEnabled(false);
			chbRotateDots->setEnabled(false);
			//slDotsize->setEnabled(true);
			break;

		case MET_PUNCHED:
			TabWidget1->setTabEnabled(ColormapPage, true);
			TabWidget1->setTabEnabled(PatternPage, false);
			ColormapGroupBox->setEnabled(false);
			PatternmapGroupBox->setEnabled(false);
			GogglesGroupBox->setEnabled(false);
			chbRotateDots->setEnabled(false);
			//slDotsize->setEnabled(true);
			break;

		case MET_PATTERNMAPPED:
			TabWidget1->setTabEnabled(ColormapPage, false);
			TabWidget1->setTabEnabled(PatternPage, true);
			ColormapGroupBox->setEnabled(false);
			PatternmapGroupBox->setEnabled(true);
			GogglesGroupBox->setEnabled(false);
			chbRotateDots->setEnabled(false);
			//slDotsize->setEnabled(false);
			break;

		case MET_PATTERNDOTTED:
			TabWidget1->setTabEnabled(ColormapPage, false);
			TabWidget1->setTabEnabled(PatternPage, true);
			ColormapGroupBox->setEnabled(false);
			PatternmapGroupBox->setEnabled(false);
			GogglesGroupBox->setEnabled(false);
			chbRotateDots->setEnabled(true);
			//slDotsize->setEnabled(false);
			break;

		default:
			break;
	}

}

/** No descriptions */
void
 KrosseyeView::ConfigureDocument(void)
{
	int mind, maxd;
	char buffer[20];
  KrosseyeDoc::lymrmeth new_lymrmeth = Doc->layer_merging_method;


	Doc->dotsize = slDotsize->value();

	/* Colormap scatter and dilutions */
	Doc->color_scatter = Scatter->value();
	Doc->color_dilution = Dilute->value() / 100.0;

	/* Depthmaop offset and maxdepth */
	mind = atoi(txtMindepth->text());
	maxd = atoi(txtMaxdepth->text());
	if (mind > 0 && mind < maxd && maxd < 1000)
	{
		Doc->mindepth = mind;
		Doc->maxdepth = maxd;
	}
	else
	{
		sprintf(buffer, "%d", Doc->mindepth);
		txtMindepth->setText(buffer);
		sprintf(buffer, "%d", Doc->maxdepth);
		txtMaxdepth->setText(buffer);

	  QMessageBox::about(this, tr("Bad Parameters"),
			  tr
			  ("Mindepth must be above zero but less than Maxdepth,\nMaxdepth must be less than 1000."));

		return;
	}
	Doc->invert_depth = cbInverted->isChecked();
	Doc->depthmap_smoothing = Smoothing->value();
	Doc->black_is_trans = cbBlackIsTrans->isChecked();
	Doc->rotate_dots = chbRotateDots->isChecked();
	Doc->horiz_handle_centered = chbHCent->isChecked();
	
	
	switch(cbNullzones->currentItem())
	{
		case NUM_IGNORE:
			Doc->nullzone_mode = KrosseyeDoc::NULM_IGNORE;
			break;
		case NUM_BRIDGE:
			Doc->nullzone_mode = KrosseyeDoc::NULM_BRIDGE;
			break;

		case NUM_OL_ALIGN:
			Doc->nullzone_mode = KrosseyeDoc::NULM_OL_ALIGN;
			break;

		case NUM_OL_MIRRORED:
			Doc->nullzone_mode = KrosseyeDoc::NULM_OL_MIRRORED;
			break;

		case NUM_OL_FAR:
			Doc->nullzone_mode = KrosseyeDoc::NULM_OL_FAR;
			break;
	}
	

//  Doc->redraw_screen_often = chbRedrawOften->isChecked();

	/*dotsize */
	Doc->dotsize = slDotsize->value();

	switch (MethodComboBox->currentItem())
	{
		case MET_RANDOMCOLORS:
		  Doc->setRenderMethod(KrosseyeDoc::REND_RANDOMCOLORS);
			break;

		case MET_COLORMAPPED:
		  Doc->setRenderMethod(KrosseyeDoc::REND_COLORMAPPED);
			break;

		case MET_PATTERNMAPPED:
		  Doc->setRenderMethod(KrosseyeDoc::REND_PATTERNMAPPED);
			break;

		case MET_SCATTERED_DOTS:
		  Doc->setRenderMethod(KrosseyeDoc::REND_SCATTERED_DOTS);
			break;

		case MET_PUNCHED:
		  Doc->setRenderMethod(KrosseyeDoc::REND_PUNCHED);
			break;

		case MET_PATTERNDOTTED:
		  Doc->setRenderMethod(KrosseyeDoc::REND_PATTERNDOTTED);
			break;

		case MET_GOGGLES:
		  Doc->setRenderMethod(KrosseyeDoc::REND_GOGGLES);
			break;
	}

	

	Doc->horiz_orgin_perc = HOrgin->value();
	Doc->vert_orgin_perc = VOrgin->value();


	switch (cbLayerMerge->currentItem())
	{
		case LMR_BRIGHTEST:
		  new_lymrmeth = KrosseyeDoc::LYMR_BRIGHTEST;
			break;
		case LMR_LATEST:
		  new_lymrmeth = KrosseyeDoc::LYMR_LATEST;
			break;
		case LMR_AVERAGE:
		  new_lymrmeth = KrosseyeDoc::LYMR_AVERAGE;
			break;
		case LMR_TRANS:
		default:
		  new_lymrmeth = KrosseyeDoc::LYMR_TRANS;
			break;

	}

	Doc->layer_merging_method = new_lymrmeth;

	switch (cbMouse->currentItem())
	{
		default:
		case MOU_NONE:
		  Doc->mouse_function = KrosseyeDoc::MOUSE_NONE;
			break;
		case MOU_RAISE:
		  Doc->mouse_function = KrosseyeDoc::MOUSE_RAISE;
			break;
		case MOU_LOWER:
		  Doc->mouse_function = KrosseyeDoc::MOUSE_LOWER;
			break;
		case MOU_SOFTEN:
		  Doc->mouse_function = KrosseyeDoc::MOUSE_SOFTEN;
			break;
		case MOU_SPREAD:
		  Doc->mouse_function = KrosseyeDoc::MOUSE_SPREAD;
			break;
		case MOU_BRIDGE1:
		  Doc->mouse_function = KrosseyeDoc::MOUSE_BRIDGE1;
			break;
		case MOU_BRIDGE2:
		  Doc->mouse_function = KrosseyeDoc::MOUSE_BRIDGE2;
			break;
		case MOU_DOT:
		  Doc->mouse_function = KrosseyeDoc::MOUSE_DOT;
			break;
	}
	Doc->mouse_brush_size = slBrushSize->value();
	Doc->mouse_brush_pow = slBrushPow->value();
}


/** No descriptions */
void
 KrosseyeView::slotRender()
{
	if (Doc->isRendering())
		return;

	ConfigureDocument();
	PrefFrame->setEnabled(false);
	Doc->render_by_buttonpress();
}


void
 KrosseyeView::slotStop()
{
	Doc->renderer_stop();
}

/** No descriptions */
void
 KrosseyeView::slotRemerge()
{
	ConfigureDocument();
	Doc->update.make_all_dirty();
	Doc->UpdateStereogram();
}



/** No descriptions */
void
 KrosseyeView::slotMousePressEvent(QMouseEvent e)
{

	ConfigureDocument();

  if (e.button() == QMouseEvent::LeftButton &&
  mouse_pressed == QMouseEvent::NoButton)
	{
		mouse_dn_x = e.x();
		mouse_dn_y = e.y();
		mouse_x = mouse_dn_x;
		mouse_y = mouse_dn_y;
		for (int y = 0; y < Doc->heigth; y++)
			Doc->horizorgin.set(y, -1);
	}
	mouse_pressed = e.button();
	
	slotMouseMoveEvent(e);

}

/** No descriptions */
void
 KrosseyeView::slotMouseReleaseEvent(QMouseEvent e)
{
	emit signalMousePos(-1, -1);
  if (mouse_pressed == QMouseEvent::LeftButton)
	{
		mouse_up_x = e.x();
		mouse_up_y = e.y();

		switch (TabWidget1->currentPageIndex())
		{
			case TAB_STEREOGRAM_PAGE_INDEX:
				if (MethodComboBox->currentItem() == MET_PATTERNMAPPED)
				{
					int last = mouse_dn_x;

					if (mouse_dn_y == mouse_up_y)
					{
						mouse_dn_y += 5;
						mouse_up_y -= 5;
						for (int y = 0; y < Doc->heigth; y++)
							Doc->horizorgin.set(y, last);

					}
					else
					{
						for (int y = 0; y < Doc->heigth; y++)
						{
							if (Doc->horizorgin.get(y) >= 0)
								last = Doc->horizorgin.get(y);

							Doc->horizorgin.set(y, last);
						}
					}


				}
				Doc->render_by_mouseclick(mouse_dn_x, mouse_dn_y, mouse_up_x, mouse_up_y);
				break;

			case TAB_DEPTHMAP_PAGE_INDEX:
				if (cbMouse->currentItem() == MOU_BRIDGE1)
					Doc->bridge1_depthmap_by_mouseclick(mouse_dn_x, mouse_dn_y, mouse_up_x, mouse_up_y);

				if (cbMouse->currentItem() == MOU_BRIDGE2)
					Doc->bridge2_depthmap_by_mouseclick(mouse_dn_x, mouse_dn_y, mouse_up_x, mouse_up_y);
				break;
		}

	}
  mouse_pressed = QMouseEvent::NoButton;
	Doc->UpdateStereogram();
	//  slotDocumentChanged();
}

/** No descriptions */
void
 KrosseyeView::slotMouseMoveEvent(QMouseEvent e)
{
	emit signalMousePos(e.x(), e.y());

	if (mouse_pressed)
	{
		switch (TabWidget1->currentPageIndex())
		{
			case TAB_STEREOGRAM_PAGE_INDEX:
			  if (mouse_pressed == QMouseEvent::LeftButton)
				{
					int x1 = mouse_x, y1 = mouse_y;
					int x2 = e.x(), y2 = e.y();

					if (y2 < 0)
						y2 = 0;
					if (y2 >= Doc->width)
						y2 = Doc->width - 1;


					mouse_x = x2;
					mouse_y = y2;

					if(MethodComboBox->currentItem() == MET_PATTERNMAPPED)
					{
							if (y2 == y1)
								y2 = y1 + 1;
							if (y2 < y1)
							{
								int i;
								i = y1;
								y1 = y2;
								y2 = i;
	
								i = x1;
								x1 = x2;
								x2 = i;
							}

							for (int y = 0; y < (y2 - y1); y++)
							{
								Doc->horizorgin.set(y1 + y, x1 + (x2 - x1) * y / (y2 - y1));
							}

					}
					else if(MethodComboBox->currentItem() != MET_PATTERNDOTTED)
					{
							Doc->render_by_mouseclick(e.x(), e.y());
					}
	
				}
				break;

			case TAB_DEPTHMAP_PAGE_INDEX:
				switch (cbMouse->currentItem())		/* FIX LATER */
				{
					default:
					case MOU_NONE:
						break;

					case MOU_RAISE:
					  if (mouse_pressed == QMouseEvent::LeftButton)
							Doc->raise_depthmap_by_mouseclick(e.x(), e.y());
						else
							Doc->lower_depthmap_by_mouseclick(e.x(), e.y());
						break;
					case MOU_LOWER:
					  if (mouse_pressed == QMouseEvent::LeftButton)
							Doc->lower_depthmap_by_mouseclick(e.x(), e.y());
						else
							Doc->raise_depthmap_by_mouseclick(e.x(), e.y());
						break;
					case MOU_SOFTEN:
					  if (mouse_pressed == QMouseEvent::LeftButton)
							Doc->soften_depthmap_by_mouseclick(e.x(), e.y());
						else
							Doc->spread_depthmap_by_mouseclick(e.x(), e.y());
						break;
					case MOU_SPREAD:
					  if (mouse_pressed == QMouseEvent::LeftButton)
							Doc->spread_depthmap_by_mouseclick(e.x(), e.y());
						else
							Doc->soften_depthmap_by_mouseclick(e.x(), e.y());
						break;
					case MOU_BRIDGE1:
					case MOU_BRIDGE2:
						break;
					case MOU_DOT:
					  if (mouse_pressed == QMouseEvent::LeftButton)
							Doc->dot_depthmap_by_mouseclick(e.x(), e.y());
						break;
				}
				break;
		}						/* tab */
	}							/* mouse pressed */
}

void KrosseyeView::slotLeftCol()
{
	QColor col;
  col = QColorDialog::getColor(Doc->goggle_left_color);
	if (col.isValid())
	{
		Doc->goggle_left_color = col;
	}

}

void KrosseyeView::slotRightCol()
{
	QColor col;
  col = QColorDialog::getColor(Doc->goggle_right_color);
	if (col.isValid())
	{
		Doc->goggle_right_color = col;
	}
}

/** No descriptions */
void
 KrosseyeView::slotProcessing(int percent_done)
{
	if (percent_done < 0 || percent_done >= 100)
	{
		PrefFrame->setEnabled(true);
	}
}
